#define ARROW_UP 233
#define ARROW_DOWN 234
#define INVISIBLE_OBJECT 32

void deleteObject(string id, int bar_index) {
	ObjectDelete(getObjectName(id, bar_index));
}

void deleteObjects(string basic_id, int max_bars_number) {
	for(int bar_index = 0; bar_index < max_bars_number; bar_index++)
		deleteObject(basic_id, bar_index);
}

void drawLabel(string id, string text, color text_color, int font_size, int corner, int horizontal_margin, int vertical_margin, int window = 0) {
	placeLabel(id, text, "Arial", text_color, font_size, corner, horizontal_margin, vertical_margin, window);
}

void drawRectangle(string basic_id, color figure_color, int corner, int left, int top, int width, int height, int window = 0) {
	ObjectCreate(basic_id, OBJ_RECTANGLE_LABEL, window, 0, 0);
	ObjectSetInteger(0, basic_id, OBJPROP_BGCOLOR, figure_color);
	
	ObjectSetInteger(0, basic_id, OBJPROP_XSIZE, width);
	ObjectSetInteger(0, basic_id, OBJPROP_YSIZE, height);
	
	ObjectSet(basic_id, OBJPROP_XDISTANCE, left);
	ObjectSet(basic_id, OBJPROP_CORNER, corner);
	ObjectSet(basic_id, OBJPROP_YDISTANCE, top);
	
	ObjectSetInteger(0, basic_id, OBJPROP_BORDER_TYPE, BORDER_FLAT);
	ObjectSetInteger(0, basic_id, OBJPROP_WIDTH, 0);
	ObjectSetInteger(0, basic_id, OBJPROP_COLOR, figure_color);
}

int getArrowCode(int value) {
	return value > 0 ? ARROW_UP : (value < 0 ? ARROW_DOWN : INVISIBLE_OBJECT);
}

void drawArrow(string id, int arrow_code, color text_color, int font_size, int corner, int horizontal_margin, int vertical_margin, int window = 0) {
	placeLabel(id, CharToStr(arrow_code), "Wingdings", text_color, font_size, corner, horizontal_margin, vertical_margin, window);
}

void createInputText(string id, string text, color text_color, ENUM_BASE_CORNER corner, int x = 0, int y = 0, int window = 0) {
	placeTextualObject(id, OBJ_EDIT, corner, x, y, window);
	ObjectSetText(id, text, "Arial", text_color);
}

void placeLabel(string id, string text, string font_family, color text_color, int font_size, int corner, int horizontal_margin, int vertical_margin, int window) {
	placeTextualObject(id, OBJ_LABEL, corner, horizontal_margin, vertical_margin, window);
	ObjectSetText(id, text, font_size, font_family, text_color);
}

void placeTextualObject(string id, int object_type, int corner, int horizontal_margin, int vertical_margin, int window) {
	ObjectCreate(id, object_type, window, 0, 0);
	ObjectSet(id, OBJPROP_XDISTANCE, horizontal_margin);
	ObjectSet(id, OBJPROP_CORNER, corner);
	ObjectSet(id, OBJPROP_YDISTANCE, vertical_margin);
}

void drawObject(string id, int bar_index, double vertical_base, int vertical_padding, int shape, color object_color, int window = 0) {
	string complete_id = id + " " + bar_index;
	ObjectCreate(complete_id, OBJ_ARROW, window, Time[bar_index], vertical_base + vertical_padding * Point);
	ObjectSet(complete_id, OBJPROP_ARROWCODE, shape);
	ObjectSet(complete_id, OBJPROP_COLOR, object_color);
}

void drawText(string id, int bar_index, double vertical_base, int vertical_padding, string text, color text_color, int font_size, string font_family = "Arial", int window = 0) {
	string complete_id = id + " " + bar_index;
	ObjectCreate(complete_id, OBJ_TEXT, window, Time[bar_index], vertical_base + vertical_padding * Point);
	ObjectSetText(complete_id, text, font_size, font_family, text_color);
}

void drawHorizontalLine(string object_name, double price, int start_bar, int end_bar, color line_color, int line_style, int window = 0) {
	drawAbstractLine(object_name, OBJ_HLINE, price, start_bar, end_bar, line_color, line_style, window);
}

void drawVerticalLine(string object_name, int bar, color line_color, int line_style, int window = 0) {
	drawAbstractLine(object_name, OBJ_VLINE, 0, bar, 0, line_color, line_style, window);
}

void drawTrendLine(string object_name, double price, int start_bar, int end_bar, color line_color, int line_style, int window = 0) {
	drawAbstractLine(object_name, OBJ_TREND, price, start_bar, end_bar, line_color, line_style, window);
	ObjectSet(object_name, OBJPROP_RAY, false);
}

void drawAbstractLine(string object_name, int object_type, double price, int start_bar, int end_bar, color line_color, int line_style, int window = 0) {
	ObjectDelete(object_name);
	ObjectCreate(object_name, object_type, window, Time[start_bar], price, Time[end_bar], price);
	ObjectSet(object_name, OBJPROP_COLOR, line_color);
	ObjectSet(object_name, OBJPROP_STYLE, line_style);
}

string getObjectName(string id, int bar_index) {
	return (id + " " + bar_index);
}

color getRgbColor(int red, int green, int blue) {
	return StringToColor(red + "," + green + "," + blue);
}

void drawChartRectangle(string object_id, int left, double top, int right, double bottom, color object_color) {
	drawDateTimeRectangle(object_id, Time[left], top, Time[right], bottom, object_color);
}

void drawDateTimeRectangle(string object_id, datetime left, double top, datetime right, double bottom, color object_color) {
	ObjectCreate(object_id, OBJ_RECTANGLE, 0, right, top, left, bottom);
	ObjectSet(object_id, OBJPROP_BACK, true);
	ObjectSet(object_id, OBJPROP_COLOR, object_color);
}

bool isButtonClicked(string button_name, int x, int y) {
	int left = ChartGetInteger(0, CHART_WIDTH_IN_PIXELS) - ObjectGetInteger(0, button_name, OBJPROP_XDISTANCE);
	int top = ObjectGetInteger(0, button_name, OBJPROP_YDISTANCE);
	int width = ObjectGetInteger(0, button_name, OBJPROP_XSIZE);
	int height = ObjectGetInteger(0, button_name, OBJPROP_YSIZE);
	return (left <= x) && (top <= y) && (left + width >= x) && (top + height >= y);
}