#property copyright "Alexander Elgin"
#property link      "alexmost113@mail.ru"
#property indicator_separate_window

#include <graphical.mqh>
#include <strings.mqh>
#include <timeframes.mqh>

#define NVO_BUFFERS 5

extern string SymbolSet = "AUDUSD, EURUSD, GBPUSD, USDJPY, USDCHF, USDCAD, NZDUSD";
extern int BarsNumber = 100;
extern int BarsBeforeExtremum = 1;
extern int BarsAfterExtremum = 2;

extern double ExtremeDifferenceCoefficient = 1.5;
extern color ColorBackgroundUp = LightGreen;
extern color ColorBackgroundStronglyUp = LimeGreen;
extern color ColorBackgroundDown = Pink;
extern color ColorBackgroundStronglyDown = Violet;
extern color ColorArrowUp = Green;
extern color ColorArrowDown = Red;
extern color ColorLabel = Blue;
extern color ColorBorder = Gray;
color color_arrow_neutral = Gold;

int corner = 0;
int font_size = 10;

int basic_horizontal_margin = 150;
int basic_vertical_margin = 25;
int horizontal_step = 70;
int vertical_step = 20;
int cell_vertical_margin = 3;
int cell_horizontal_margin = 9;
int label_horizontal_margin = 25;

string indicator_name = "NVO Dashboard";
string symbols[];
int timeframes[8] = {PERIOD_M1, PERIOD_M5, PERIOD_M15, PERIOD_M30, PERIOD_H1, PERIOD_H4, PERIOD_D1, PERIOD_W1};
int bars[2] = {1, 0};

int init() {
	IndicatorShortName(indicator_name);
	SetIndexLabel(0, indicator_name);

	split(SymbolSet, ",", symbols);
	
	drawTable();
	drawHorizontalBorders();
	drawVerticalBorders();
	drawVerticalSubBorders();
	return 0;
}

int start() {
	drawTable();
	return 0;
}

int deinit() {
	deleteHorizontalBorders();
	deleteVerticalBorders();
	deleteVerticalSubBorders();

	deleteTimeframes();
	deleteSymbols();
	for(int row = 0; row < ArraySize(symbols); row++)
		deleteSymbolValues(symbols[row]);
	return(0);
}

void drawHorizontalBorders() {
	for(int row = -1; row <= ArraySize(symbols) + 1; row++)
		drawRectangle("horizontal_border" + row, ColorBorder, corner, basic_horizontal_margin - horizontal_step, basic_vertical_margin + row * vertical_step, horizontal_step + ArraySize(timeframes) * horizontal_step, 1, WindowFind(indicator_name));
}

void deleteHorizontalBorders() {
	for(int row = 0; row <= ArraySize(symbols); row++)
		ObjectDelete("horizontal_border" + row);
}

void drawVerticalBorders() {
	for(int column = -1; column <= ArraySize(timeframes); column++)
		drawRectangle("vertical_border" + column, ColorBorder, corner, basic_horizontal_margin + column * horizontal_step, basic_vertical_margin - vertical_step, 1, (ArraySize(symbols) + 2) * vertical_step, WindowFind(indicator_name));
}

void deleteVerticalBorders() {
	for(int column = -1; column <= ArraySize(timeframes); column++)
		ObjectDelete("vertical_border" + column);
}

void drawVerticalSubBorders() {
	for(int column = 0; column < ArraySize(timeframes); column++)
		drawRectangle("vertical_sub_border" + column, ColorBorder, corner, basic_horizontal_margin + (column + 0.5) * horizontal_step, basic_vertical_margin, 1, (ArraySize(symbols) + 1) * vertical_step, WindowFind(indicator_name));
}

void deleteVerticalSubBorders() {
	for(int column = 0; column <= ArraySize(timeframes); column++)
		ObjectDelete("vertical_sub_border" + column);
}

void drawTable() {
	drawTimeframes();
	for(int row = 0; row < ArraySize(symbols); row++)
		drawSymbolValues(symbols[row], row);
}

void deleteSymbolValues(string symbol) {
	for(int column = 0; column < ArraySize(timeframes); column++)
		for(int bar_index = 0; bar_index < ArraySize(bars); bar_index++)
			deleteCell((symbol + timeframes[column]) + bars[bar_index]);
}

void deleteCell(string label_name) {
	ObjectDelete(label_name);
	ObjectDelete(label_name + "background");
}

void deleteSymbols() {
	for(int row = 0; row < ArraySize(symbols); row++)
		ObjectDelete(symbols[row] + "name");
}

void drawTimeframes() {
	for(int column = 0; column < ArraySize(timeframes); column++) {
		int left = basic_horizontal_margin + column * horizontal_step;
		drawLabel("period" + timeframes[column], getTimeframeName(timeframes[column]), ColorLabel, font_size, corner, left + label_horizontal_margin, basic_vertical_margin - vertical_step + 2, WindowFind(indicator_name));
		
		for(int bar_index = 0; bar_index < ArraySize(bars); bar_index++)
			drawLabel(("bar" + timeframes[column]) + bars[bar_index], bars[bar_index], ColorLabel, font_size, corner, left + bar_index * horizontal_step / 2 + cell_horizontal_margin, basic_vertical_margin, WindowFind(indicator_name));
	}
}

void deleteTimeframes() {
	for(int column = 0; column < ArraySize(timeframes); column++) {
		ObjectDelete("period" + timeframes[column]);
		for(int bar_index = 0; bar_index < ArraySize(bars); bar_index++)
			ObjectDelete(("bar" + timeframes[column]) + bars[bar_index]);
	}
}

void drawSymbolValues(string symbol, int row) {
	int top = basic_vertical_margin + (row + 1) * vertical_step;
	
	for(int column = 0; column < ArraySize(timeframes); column++) {
		int window = WindowFind(indicator_name);
		
		for(int bar_index = 0; bar_index < ArraySize(bars); bar_index++) {
			int extremum_bar;
			
			int bottom_bar = getBottomBar(symbol, timeframes[column], bars[bar_index] + BarsAfterExtremum);
			bool bottom_breakout = bottom_bar != -1;
			for(extremum_bar = bottom_bar - 1; extremum_bar > bars[bar_index]; extremum_bar--)
				if(isBottomBreakout(symbol, timeframes[column], bottom_bar, extremum_bar)) bottom_breakout = false;
			bottom_breakout = bottom_breakout && isBottomBreakout(symbol, timeframes[column], bottom_bar, bars[bar_index]);
			
			int top_bar = getTopBar(symbol, timeframes[column], bars[bar_index] + BarsAfterExtremum);
			bool top_breakout = top_bar != -1;
			for(extremum_bar = top_bar - 1; extremum_bar > bars[bar_index]; extremum_bar--)
				if(isTopBreakout(symbol, timeframes[column], top_bar, extremum_bar)) top_breakout = false;
			top_breakout = top_breakout && isTopBreakout(symbol, timeframes[column], top_bar, bars[bar_index]);
			
			string label_name = (symbol + timeframes[column]) + bars[bar_index];
			int value = top_breakout ? 1 : (bottom_breakout ? -1 : 0);
			
			if(value) {
				int left = basic_horizontal_margin + (2 * column + bar_index) * horizontal_step / 2;
				drawRectangle(label_name + "background", getBackgroundColor(symbol, timeframes[column], bars[bar_index]), corner, left, top, horizontal_step / 2, vertical_step, window);
				drawArrow(label_name, getArrowCode(value), getColor(value), font_size, corner, left + cell_horizontal_margin, top + cell_vertical_margin, window);
			} else
				deleteCell(label_name);
		}
	}
	
	drawLabel(symbol + "name", symbol, ColorLabel, font_size, corner, basic_horizontal_margin - horizontal_step + 5, top + 2, WindowFind(indicator_name));
}

bool isBottomBreakout(string symbol, int period, int bottom_bar, int bar) {
	return iClose(symbol, period, bar) < iClose(symbol, period, bottom_bar);
}

bool isTopBreakout(string symbol, int period, int top_bar, int bar) {
	return iClose(symbol, period, bar) > iClose(symbol, period, top_bar);
}

int getTopBar(string symbol, int period, int current_bar) {
	for(int bar = current_bar; bar < BarsNumber; bar++)
		if(isTop(symbol, period, bar)) return bar;
	return -1;
}

int getBottomBar(string symbol, int period, int current_bar) {
	for(int bar = current_bar; bar < BarsNumber; bar++)
		if(isBottom(symbol, period, bar)) return bar;
	return -1;
}

bool isTop(string symbol, int period, int bar) {
	return isMaximum(symbol, period, bar, 1, BarsBeforeExtremum) && isMaximum(symbol, period, bar, -1, BarsAfterExtremum);
}

bool isBottom(string symbol, int period, int bar) {
	return isMinimum(symbol, period, bar, 1, BarsBeforeExtremum) && isMinimum(symbol, period, bar, -1, BarsAfterExtremum);
}

bool isMaximum(string symbol, int period, int current_bar, int direction, int bars_number) {
	for(int bar = 1; bar <= bars_number; bar++)
		if(iClose(symbol, period, current_bar + direction * bar) > iClose(symbol, period, current_bar))
			return false;
	return true;
}

bool isMinimum(string symbol, int period, int current_bar, int direction, int bars_number) {
	for(int bar = 1; bar <= bars_number; bar++)
		if(iClose(symbol, period, current_bar + direction * bar) < iClose(symbol, period, current_bar))
			return false;	
	return true;
}

color getColor(double value) {
	return value > 0 ? ColorArrowUp : (value < 0 ? ColorArrowDown : color_arrow_neutral);
}

color getBackgroundColor(string symbol, int period, int bar) {
	double current_volume = iVolume(symbol, period, bar);
	double previous_volume = iVolume(symbol, period, bar + 1);
	bool is_difference_extreme = MathMax(current_volume, previous_volume) >= ExtremeDifferenceCoefficient * MathMin(current_volume, previous_volume);
	
	return current_volume > previous_volume
		? (is_difference_extreme ? ColorBackgroundStronglyUp : ColorBackgroundUp)
		: (is_difference_extreme ? ColorBackgroundStronglyDown : ColorBackgroundDown);
}

int getSymbolsNumber() {
	return (ArraySize(symbols));
}